/****************************************************************************
  This file is part of the Freedom Remailer.  The encrypt_sub() function
  is borrowed from Andy Dustman's Encrypt-Subject patch to Matt Ghio's
  remailer distribution.  All changes and other code are:
  Copyright (C) 1997  Johannes Kroeger (hanne@squirrel.owl.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <syslog.h>
#include "idea.h"
#include "md5.h"
#include "freedom.h"

int
scan_subject(const char *bodyfile)
{
  int found_subject = 0;
  FILE *in;
  char line[BUFSIZ], tmpline[BUFSIZ];

  in = fopen(bodyfile, "r");
  while (fgets(line, sizeof(line), in)) {
    if (strlen(line) <= 1) {
      fclose(in);
      return found_subject;
    }
    else {
      if (strileft(line, "subject:")) {
	found_subject++;
	strcpy (tmpline, line + sizeof("subject:") - 1);
	chop(tmpline);
	if (blocked(tmpline, SUBJ_BLOCK)) {
	  fclose(in);
	  return -1;
	}
      }
    }
  }
  /* no blank line found */
  fclose(in);
  return found_subject;
}

void
encrypt_sub(const char *origfilename, const char *key)
/* IDEA-encrypt MD5-hash of ##'ed Subject: line with MD5-hash of "key" */
{
  FILE *infile, *outfile;
  int headers = 1, colhead = 0, addhead = 0, linenum = 0, x;
  char tempfilename[BUFSIZ], line[BUFSIZ], tmpline[BUFSIZ];
  struct MD5Context subjkeyDigest, subjectDigest;
  struct IdeaCfbContext subjcontext;
  unsigned char subjIDEAkey[16], subjPT[16];
  unsigned char subjCT[24]; /* includes IV */
  char hexrand[9];

  MD5Init(&subjkeyDigest);
  MD5Update(&subjkeyDigest, key, strlen(key));
  MD5Final(subjIDEAkey, &subjkeyDigest);

  sprintf(tempfilename, "%sS", origfilename);
  rename(origfilename, tempfilename);

  if (!(infile = fopen(tempfilename, "r"))) {
    outfile = fopen(origfilename, "w");
    fclose(outfile);
    return;
  }
  outfile = fopen(origfilename, "w");

  while (fgets(line, sizeof(line), infile)) {
    if (!headers) {
      if ((linenum > colhead) || (strlen(line) > 1))
	linenum++;
      if ((linenum == 1) && strstr(line, "::")) {
	headers = 1;
	colhead = 1;
	fputs(line, outfile);
      }
      else if ((linenum == (colhead + 1)) && strstr(line, "##")) {
	addhead = 1;
	fputs(line, outfile);
      }
      else {
	if (addhead) {
	  if (strlen(line) <= 1)
	    addhead = 0;
	  if (!strileft(line, "subject:"))
	    fputs(line, outfile);
	  else {
	    if (USE_SYSLOG)
	      syslog(LOG_DEBUG, "Encrypting %s", line);
	    strcpy(tmpline, line + sizeof("subject:") - 1);
	    chop(tmpline);
	    MD5Init(&subjectDigest);
	    MD5Update(&subjectDigest, tmpline, strlen(tmpline));
	    MD5Final(subjPT, &subjectDigest);
	    ideaCfbInit(&subjcontext, subjIDEAkey);
	    sprintf(hexrand, "%08x", rand());
	    memcpy(subjCT, hexrand, 8);
	    memcpy(subjCT + 8, subjPT, 16) ;
	    ideaCfbEncrypt(&subjcontext, subjCT, subjCT, 24);
	    ideaCfbDestroy(&subjcontext);
	    fprintf(outfile, "Subject: ");
	    for (x = 0; x < 24; x++)
	      fprintf(outfile, "%2.2hx", subjCT[x]);
	    fprintf(outfile, "\n");
	  }
	}
	else {
	  if ((linenum > colhead) || (strlen(line) > 1))
	    fputs(line, outfile);
	}
      }
    }
    else {
      if (strlen(line) <= 1)
	headers = 0;
      if (!strileft(line, "encrypt-subject:"))
	fputs(line, outfile);
    }
  }
  fclose(infile);
  fclose(outfile);
  unlink(tempfilename);
}
